searchData={"content_type":"text/markdown","items":[{"doc":"A container for Elixir lists, that can be used to iterate forward\nand backward, with a focused \"current\" item in the list, and \"prev\" and \"next\"\nlists of items that come before and after the current item.\n\nIn Elm this has been described as a \"Zipper\".","ref":"Dmp.Cursor.html","title":"Dmp.Cursor","type":"module"},{"doc":"Returns `true` if the Cursor is positioned after the last item.","ref":"Dmp.Cursor.html#after_last?/1","title":"Dmp.Cursor.after_last?/1","type":"function"},{"doc":"iex> Cursor.from_list([1, 2]) |> Cursor.after_last?()\n    false\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(2) |> Cursor.after_last?()\n    false\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(3) |> Cursor.after_last?()\n    true","ref":"Dmp.Cursor.html#after_last?/1-examples","title":"Examples - Dmp.Cursor.after_last?/1","type":"function"},{"doc":"Returns `true` if the Cursor is positioned before the first item.","ref":"Dmp.Cursor.html#before_first?/1","title":"Dmp.Cursor.before_first?/1","type":"function"},{"doc":"iex> Cursor.from_list([1, 2, 3]) |> Cursor.before_first?()\n    true\n\n    iex> Cursor.from_list([1, 2, 3]) |> Cursor.move_forward() |> Cursor.before_first?()\n    false","ref":"Dmp.Cursor.html#before_first?/1-examples","title":"Examples - Dmp.Cursor.before_first?/1","type":"function"},{"doc":"Returns the total number of items in the Cursor.","ref":"Dmp.Cursor.html#count/1","title":"Dmp.Cursor.count/1","type":"function"},{"doc":"iex> Cursor.from_list([1, 2, 3]) |> Cursor.count()\n    3","ref":"Dmp.Cursor.html#count/1-examples","title":"Examples - Dmp.Cursor.count/1","type":"function"},{"doc":"Remove items at the Cursor's current position, leaving the previous items alone.","ref":"Dmp.Cursor.html#delete/2","title":"Dmp.Cursor.delete/2","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.delete(1)\n    %Cursor{current: 4, next: [5], prev: [2, 1]}","ref":"Dmp.Cursor.html#delete/2-examples","title":"Examples - Dmp.Cursor.delete/2","type":"function"},{"doc":"Remove items before the Cursor's current position, leaving the current and next items alone.","ref":"Dmp.Cursor.html#delete_before/2","title":"Dmp.Cursor.delete_before/2","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.delete_before(1)\n    %Cursor{current: 3, next: [4, 5], prev: [1]}","ref":"Dmp.Cursor.html#delete_before/2-examples","title":"Examples - Dmp.Cursor.delete_before/2","type":"function"},{"doc":"Returns `true` if there are no items in the Cursor.","ref":"Dmp.Cursor.html#empty?/1","title":"Dmp.Cursor.empty?/1","type":"function"},{"doc":"iex> Cursor.from_list([]) |> Cursor.empty?()\n    true\n\n    iex> Cursor.from_list([1, 2, 3]) |> Cursor.empty?()\n    false","ref":"Dmp.Cursor.html#empty?/1-examples","title":"Examples - Dmp.Cursor.empty?/1","type":"function"},{"doc":"Moves the position of the Cursor back through the \"prev\" list\nuntil the given item is found. Returns `nil` and if the item cannot be found.","ref":"Dmp.Cursor.html#find_back/2","title":"Dmp.Cursor.find_back/2","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.find_back(1)\n    %Cursor{current: 1, next: [2, 3, 4, 5], prev: []}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.find_back(5)\n    nil\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.find_back(3)\n    nil","ref":"Dmp.Cursor.html#find_back/2-examples","title":"Examples - Dmp.Cursor.find_back/2","type":"function"},{"doc":"Moves the position of the Cursor back through the \"prev\" list to the given item.\n\nRaises if the item cannot be found.","ref":"Dmp.Cursor.html#find_back!/2","title":"Dmp.Cursor.find_back!/2","type":"function"},{"doc":"Moves the position of the Cursor forward through the \"next\" list\nuntil the given item is found. Returns `nil` and if the item cannot be found.","ref":"Dmp.Cursor.html#find_forward/2","title":"Dmp.Cursor.find_forward/2","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.find_forward(5)\n    %Cursor{current: 5, next: [], prev: [4, 3, 2, 1]}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.find_forward(1)\n    nil\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.find_forward(3)\n    nil","ref":"Dmp.Cursor.html#find_forward/2-examples","title":"Examples - Dmp.Cursor.find_forward/2","type":"function"},{"doc":"Create a Cursor from a list of items.\n\n  * `opts` - The keyword `:position` can specify a position value,\n    such as `-1`, `0`, `1`, `:last`, or `tail` in order to set the\n    position of the created Cursor.","ref":"Dmp.Cursor.html#from_list/2","title":"Dmp.Cursor.from_list/2","type":"function"},{"doc":"iex> Cursor.from_list([1, 2, 3])\n    %Cursor{current: nil, next: [1, 2, 3], prev: []}\n\n    iex> Cursor.from_list([1, 2, 3]) |> Cursor.move_forward()\n    %Cursor{current: 1, next: [2, 3], prev: []}\n\n    iex> Cursor.from_list([1, 2, 3], position: 1)\n    %Cursor{current: 2, next: [3], prev: [1]}","ref":"Dmp.Cursor.html#from_list/2-examples","title":"Examples - Dmp.Cursor.from_list/2","type":"function"},{"doc":"Create a Cursor from two lists.\n\n  * `opts` - the keyword `:position` can specify a position value,\n    such as `-1`, `0`, `1`, `:last`, or `tail` in order to set the\n    position of the created Cursor.","ref":"Dmp.Cursor.html#from_split/3","title":"Dmp.Cursor.from_split/3","type":"function"},{"doc":"iex> Cursor.from_split([1, 2, 3], [4, 5, 6])\n    %Cursor{current: 4, next: [5, 6], prev: [3, 2, 1]}","ref":"Dmp.Cursor.html#from_split/3-examples","title":"Examples - Dmp.Cursor.from_split/3","type":"function"},{"doc":"Return a 3-tuple of the previous, current, and next items relative to\nthe Cursor's current position.\n\nAny elements of the tuple may be `nil`.","ref":"Dmp.Cursor.html#get/1","title":"Dmp.Cursor.get/1","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.get()\n    {2, 3, 4}\n\n    iex> Cursor.from_list([]) |> Cursor.get()\n    {nil, nil, nil}\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.get()\n    {nil, nil, 1}\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward() |> Cursor.get()\n    {nil, 1, 2}\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(2) |> Cursor.get()\n    {1, 2, nil}\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(3) |> Cursor.get()\n    {2, nil, nil}","ref":"Dmp.Cursor.html#get/1-examples","title":"Examples - Dmp.Cursor.get/1","type":"function"},{"doc":"Returns `false` if the Cursor is positioned at or after the last item.","ref":"Dmp.Cursor.html#has_next?/1","title":"Dmp.Cursor.has_next?/1","type":"function"},{"doc":"iex> Cursor.from_list([1, 2]) |> Cursor.has_next?()\n    true\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(2) |> Cursor.has_next?()\n    false\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(3) |> Cursor.has_next?()\n    false","ref":"Dmp.Cursor.html#has_next?/1-examples","title":"Examples - Dmp.Cursor.has_next?/1","type":"function"},{"doc":"Returns `false` if the Cursor is positioned at or before the first item.","ref":"Dmp.Cursor.html#has_previous?/1","title":"Dmp.Cursor.has_previous?/1","type":"function"},{"doc":"iex> Cursor.from_list([1, 2]) |> Cursor.has_previous?()\n    false\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward() |> Cursor.has_previous?()\n    false\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(2) |> Cursor.has_previous?()\n    true\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(3) |> Cursor.has_previous?()\n    true","ref":"Dmp.Cursor.html#has_previous?/1-examples","title":"Examples - Dmp.Cursor.has_previous?/1","type":"function"},{"doc":"Insert items at the Cursor's current position, leaving the previous items alone.\n\nAfter the insertion, the Cursor points to the first inserted item.","ref":"Dmp.Cursor.html#insert/2","title":"Dmp.Cursor.insert/2","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.insert([10, 11])\n    %Cursor{current: 10, next: [11, 3, 4, 5], prev: [2, 1]}","ref":"Dmp.Cursor.html#insert/2-examples","title":"Examples - Dmp.Cursor.insert/2","type":"function"},{"doc":"Insert items at the Cursor's head position, leaving the current position pointer alone.","ref":"Dmp.Cursor.html#insert_at_head/2","title":"Dmp.Cursor.insert_at_head/2","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.insert_at_head([10, 11])\n    %Cursor{current: 3, next: [4, 5], prev: [2, 1, 11, 10]}","ref":"Dmp.Cursor.html#insert_at_head/2-examples","title":"Examples - Dmp.Cursor.insert_at_head/2","type":"function"},{"doc":"Insert items before the Cursor's current position, leaving the current and next items alone.","ref":"Dmp.Cursor.html#insert_before/2","title":"Dmp.Cursor.insert_before/2","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.insert_before([10, 11])\n    %Cursor{current: 3, next: [4, 5], prev: [11, 10, 2, 1]}","ref":"Dmp.Cursor.html#insert_before/2-examples","title":"Examples - Dmp.Cursor.insert_before/2","type":"function"},{"doc":"Moves the position of the Cursor back a number of steps.","ref":"Dmp.Cursor.html#move_back/2","title":"Dmp.Cursor.move_back/2","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_back(1)\n    %Cursor{current: 2, next: [3, 4, 5], prev: [1]}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_back(2)\n    %Cursor{current: 1, next: [2, 3, 4, 5], prev: []}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_back(3)\n    %Cursor{current: nil, next: [1, 2, 3, 4, 5], prev: []}","ref":"Dmp.Cursor.html#move_back/2-examples","title":"Examples - Dmp.Cursor.move_back/2","type":"function"},{"doc":"Moves the position of the Cursor to the first item.\nAn alias of `Cursor.move_to(c, 0)`.","ref":"Dmp.Cursor.html#move_first/1","title":"Dmp.Cursor.move_first/1","type":"function"},{"doc":"iex> Cursor.new() |> Cursor.move_first()\n    %Cursor{current: nil, next: [], prev: []}\n\n    iex> Cursor.from_list([1]) |> Cursor.move_first()\n    %Cursor{current: 1, next: [], prev: []}\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_first()\n    %Cursor{current: 1, next: [2], prev: []}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_first()\n    %Cursor{current: 1, next: [2, 3, 4, 5], prev: []}","ref":"Dmp.Cursor.html#move_first/1-examples","title":"Examples - Dmp.Cursor.move_first/1","type":"function"},{"doc":"Moves the position of the Cursor forward a number of steps.","ref":"Dmp.Cursor.html#move_forward/2","title":"Dmp.Cursor.move_forward/2","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_forward()\n    %Cursor{current: 4, next: [5], prev: [3, 2, 1]}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_forward(2)\n    %Cursor{current: 5, next: [], prev: [4, 3, 2, 1]}\n\n    iex> Cursor.from_list([1, 2, 3, 4, 5]) |> Cursor.move_forward(3)\n    %Cursor{current: 3, next: [4, 5], prev: [2, 1]}\n\n    iex> Cursor.from_list([1, 2, 3, 4, 5]) |> Cursor.move_forward(5)\n    %Cursor{current: 5, next: [], prev: [4, 3, 2, 1]}\n\n    iex> Cursor.from_list([1, 2, 3, 4, 5]) |> Cursor.move_forward(6)\n    %Cursor{current: nil, next: [], prev: [5, 4, 3, 2, 1]}\n\n    iex> %Cursor{current: {:delete, \"abc\"}, next: [{:equal, \"xxx\"}, {:insert, \"def\"}], prev: []} |> Cursor.move_forward(1)\n    %Cursor{current: {:equal, \"xxx\"}, next: [{:insert, \"def\"}], prev: [{:delete, \"abc\"}]}\n\n    iex> %Cursor{current: {:delete, \"abc\"}, next: [{:equal, \"xxx\"}, {:insert, \"def\"}], prev: []} |> Cursor.move_forward(2)\n    %Cursor{current: {:insert, \"def\"}, next: [], prev: [{:equal, \"xxx\"}, {:delete, \"abc\"}]}\n\n    iex> %Cursor{current: {:delete, \"abc\"}, next: [{:equal, \"xxx\"}, {:insert, \"def\"}], prev: []} |> Cursor.move_forward(3)\n    %Cursor{current: nil, next: [], prev: [{:insert, \"def\"}, {:equal, \"xxx\"}, {:delete, \"abc\"}]}","ref":"Dmp.Cursor.html#move_forward/2-examples","title":"Examples - Dmp.Cursor.move_forward/2","type":"function"},{"doc":"Changes the current position of the Cursor.\n\n  * `pos` - The desired position.\n\n  * `-1` means the Cursor is positioned before the first item.\n  * `0` means the Cursor is positioned at the first item (if it is not empty).\n  * `:last` means the Cursor is positioned at the last item (if it is not empty).\n  * `:tail` means the Cursor is positioned after the last item.","ref":"Dmp.Cursor.html#move_to/2","title":"Dmp.Cursor.move_to/2","type":"function"},{"doc":"iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_to(-1)\n    %Cursor{current: nil, next: [1, 2, 3, 4, 5], prev: []}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_to(0)\n    %Cursor{current: 1, next: [2, 3, 4, 5], prev: []}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_to(1)\n    %Cursor{current: 2, next: [3, 4, 5], prev: [1]}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_to(:last)\n    %Cursor{current: 5, next: [], prev: [4, 3, 2, 1]}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.move_to(:tail)\n    %Cursor{current: nil, next: [], prev: [5, 4, 3, 2, 1]}","ref":"Dmp.Cursor.html#move_to/2-examples","title":"Examples - Dmp.Cursor.move_to/2","type":"function"},{"doc":"Create a Cursor containing no items.","ref":"Dmp.Cursor.html#new/0","title":"Dmp.Cursor.new/0","type":"function"},{"doc":"iex> Cursor.new()\n    %Cursor{current: nil, next: [], prev: []}","ref":"Dmp.Cursor.html#new/0-examples","title":"Examples - Dmp.Cursor.new/0","type":"function"},{"doc":"Returns the current position of the Cursor.\n\nA return value of `-1` means the Cursor is positioned before the first item.\nA return value of `Cursor.count(c)` means the Cursor is positioned after the last item.","ref":"Dmp.Cursor.html#position/1","title":"Dmp.Cursor.position/1","type":"function"},{"doc":"iex> Cursor.from_list([1, 2]) |> Cursor.position()\n    -1\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward() |> Cursor.position()\n    0\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(2) |> Cursor.position()\n    1\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(3) |> Cursor.position()\n    2\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward(4) |> Cursor.position()\n    2","ref":"Dmp.Cursor.html#position/1-examples","title":"Examples - Dmp.Cursor.position/1","type":"function"},{"doc":"Resets the position of the Cursor to before the first item.\nAn alias for `Cursor.move_to(c, -1)`.","ref":"Dmp.Cursor.html#reset/1","title":"Dmp.Cursor.reset/1","type":"function"},{"doc":"iex> Cursor.new() |> Cursor.reset()\n    %Cursor{current: nil, next: [], prev: []}\n\n    iex> Cursor.from_list([1]) |> Cursor.move_forward() |> Cursor.reset()\n    %Cursor{current: nil, next: [1], prev: []}\n\n    iex> Cursor.from_list([1, 2]) |> Cursor.move_forward() |> Cursor.reset()\n    %Cursor{current: nil, next: [1, 2], prev: []}\n\n    iex> %Cursor{current: 3, next: [4, 5], prev: [2, 1]} |> Cursor.reset()\n    %Cursor{current: nil, next: [1, 2, 3, 4, 5], prev: []}","ref":"Dmp.Cursor.html#reset/1-examples","title":"Examples - Dmp.Cursor.reset/1","type":"function"},{"doc":"Extract the list from a Cursor.","ref":"Dmp.Cursor.html#to_list/1","title":"Dmp.Cursor.to_list/1","type":"function"},{"doc":"iex> Cursor.from_list([1, 2, 3]) |> Cursor.to_list()\n    [1, 2, 3]","ref":"Dmp.Cursor.html#to_list/1-examples","title":"Examples - Dmp.Cursor.to_list/1","type":"function"},{"doc":"","ref":"Dmp.Cursor.html#t:init_option/0","title":"Dmp.Cursor.init_option/0","type":"type"},{"doc":"Use `position: 0` for example, to set the initial\nposition of the Cursor to the first item.","ref":"Dmp.Cursor.html#t:init_options/0","title":"Dmp.Cursor.init_options/0","type":"type"},{"doc":"A value used to set the Cursor position.\n\n  * `-1` means before the first item in the container.\n  * `0` means at the first item.\n  * `:last` means at the last item.\n  * `:tail` means after the last item.","ref":"Dmp.Cursor.html#t:position_value/0","title":"Dmp.Cursor.position_value/0","type":"type"},{"doc":"","ref":"Dmp.Cursor.html#t:t/0","title":"Dmp.Cursor.t/0","type":"type"},{"doc":"Utilities for debugging bitarrays.","ref":"Dmp.DebugUtils.html","title":"Dmp.DebugUtils","type":"module"},{"doc":"Returns a list of \"codepoints\" (single-character strings)\nshowing the base-2 value of `value`.\n\nA minimum of `padding` elements are returned.","ref":"Dmp.DebugUtils.html#bitmap_to_list/2","title":"Dmp.DebugUtils.bitmap_to_list/2","type":"function"},{"doc":"Formats an alphabet bitarray into a list of lines, showing binary values.\n\n* `s` - The alphabet as returned by `Dmp.Match.alphabet/1`.\n* `pattern` - The pattern that is associated with the alphabet.","ref":"Dmp.DebugUtils.html#debug_alphabet/2","title":"Dmp.DebugUtils.debug_alphabet/2","type":"function"},{"doc":"iex> DebugUtils.debug_alphabet(%{?a => 5, ?b => 2}, \"aba\")\n    [\n      \"alphabet: a b a\",\n      \"    a  5: 1 0 1\",\n      \"    b  2: 0 1 0\"\n    ]","ref":"Dmp.DebugUtils.html#debug_alphabet/2-examples","title":"Examples - Dmp.DebugUtils.debug_alphabet/2","type":"function"},{"doc":"Formats the `rd` bitarray into a list of lines, showing binary values.\n\n* `rd` - The bitarrary.\n* `text` - The text associated with `rd`.\n* `pattern` - The pattern associated with `rd`.\n* `d` - The error level.\n* `start` - The lowest index that has been calculated. Lines below this\n  index will be marked with an asterisk.\n* `best_loc` - The index in the text where the best match has been found.\n  The line at this index will be marked with an \"@\"-sign.","ref":"Dmp.DebugUtils.html#debug_rd/6","title":"Dmp.DebugUtils.debug_rd/6","type":"function"},{"doc":"iex> DebugUtils.debug_rd(%{1 => 5, 2 => 7, -1 => 3}, \"abc\", \"add\", 0, 1, 2)\n    [\n      \"  rd_j^0: a d d\",\n      \" 0* _  0: 0 0 0\",\n      \" 1  a  5: 1 0 1\",\n      \" 2  b  7: 1 1 1\",\n      \" 3@ c  0: 0 0 0\",\n      \" 4  _  0: 0 0 0\"\n    ]","ref":"Dmp.DebugUtils.html#debug_rd/6-examples","title":"Examples - Dmp.DebugUtils.debug_rd/6","type":"function"},{"doc":"Compare two blocks of plain text and efficiently return a list of differences.","ref":"Dmp.Diff.html","title":"Dmp.Diff","type":"module"},{"doc":"Find the \"middle snake\" of a diff, split the problem in two\nand return the recursively constructed diff.\n\nSee: [An O(ND) Difference Algorithm and Its Variations (Meyers, 1986)](http://www.xmailserver.org/diff2.pdf)\n\n  * `text1` - Old string to be diffed.\n  * `text2` - New string to be diffed.\n  * `deadline` - Unix timestamp (in milliseconds) at which to bail if not yet complete.\n\nReturns a difflist.","ref":"Dmp.Diff.html#bisect/3","title":"Dmp.Diff.bisect/3","type":"function"},{"doc":"Given the location of the \"middle snake\", split the diff in two parts\nand recurse.\n\n  * `text1` - Old string to be diffed.\n  * `text2` - New string to be diffed.\n  * `x` - Index of split point in text1.\n  * `y` - Index of split point in text2.\n  * `deadline` - Unix timestamp (in milliseconds) at which to bail if not yet complete.\n\nReturns a difflist.","ref":"Dmp.Diff.html#bisect_split/5","title":"Dmp.Diff.bisect_split/5","type":"function"},{"doc":"Rehydrate the text in a diff from a string of line hashes to real lines of\ntext.\n\n  * `diffs` - A difflist.\n  * `line_array` - A list of unique strings.\n\nReturns the rehydrated difflist.","ref":"Dmp.Diff.html#chars_to_lines/2","title":"Dmp.Diff.chars_to_lines/2","type":"function"},{"doc":"Reduce the number of edits in a diff by eliminating operationally trivial equalities.\n\n  * `diff_edit_cost`  Cost of an empty edit operation in terms of edit characters.\n\nReturns the updated difflist.","ref":"Dmp.Diff.html#cleanup_efficiency/2","title":"Dmp.Diff.cleanup_efficiency/2","type":"function"},{"doc":"Reorder and merge like edit sections in a diff, merging equalities.\n\nAny edit section can move as long as it doesn't cross an equality.\n\nReturns the updated difflist.","ref":"Dmp.Diff.html#cleanup_merge/1","title":"Dmp.Diff.cleanup_merge/1","type":"function"},{"doc":"Reduce the number of edits in a diff by eliminating semantically trivial equalities.\n\nReturns the updated difflist.","ref":"Dmp.Diff.html#cleanup_semantic/1","title":"Dmp.Diff.cleanup_semantic/1","type":"function"},{"doc":"Look for single edits in a diff that are surrounded on both sides by equalities\nwhich can be shifted sideways to align the edit to a word boundary.\n\nExample: `The c at c ame.` becomes `The  cat  came.`\n\nReturns the updated difflist.","ref":"Dmp.Diff.html#cleanup_semantic_lossless/1","title":"Dmp.Diff.cleanup_semantic_lossless/1","type":"function"},{"doc":"","ref":"Dmp.Diff.html#combine_previous_inequalities/6","title":"Dmp.Diff.combine_previous_inequalities/6","type":"function"},{"doc":"Determine if the suffix of one string is the prefix of another.\n\n  * `text1` - First string.\n  * `text2` - Second string.\n\nReturns the number of characters common to the end of the first\nstring and the start of the second string.","ref":"Dmp.Diff.html#common_overlap/2","title":"Dmp.Diff.common_overlap/2","type":"function"},{"doc":"Determine the common prefix of two strings.\n\n  * `text1` - First string.\n  * `text2` - Second string.\n\nReturns a tuple `{prefix, rest1, rest2}`, where\n\n  * `prefix` - The common prefix.\n  * `rest1` - `text1` with the prefix removed.\n  * `rest2` - `text2` with the prefix removed.","ref":"Dmp.Diff.html#common_prefix/2","title":"Dmp.Diff.common_prefix/2","type":"function"},{"doc":"Determine the common suffix of two strings.\n\n  * `text1` - First string.\n  * `text2` - Second string.\n\nReturns a tuple `{suffix, rest1, rest2}`, where\n\n  * `suffix` - The common suffix.\n  * `rest1` - `text1` with the suffix removed.\n  * `rest2` - `text2` with the suffix removed.","ref":"Dmp.Diff.html#common_suffix/2","title":"Dmp.Diff.common_suffix/2","type":"function"},{"doc":"Find the differences between two texts.\n\n  * `text1` - Old string to be diffed.\n  * `text2` -  New string to be diffed.\n  * `check_lines` - Speedup flag.  If false, then don't run a\n    line-level diff first to identify the changed areas.\n    If true, then run a faster slightly less optimal diff.\n  * `deadline` - Unix timestamp in milliseconds when the diff should be complete by.\n\nAssumes that the texts do not have any common prefix or suffix.\n\nReturns a difflist.","ref":"Dmp.Diff.html#compute/4","title":"Dmp.Diff.compute/4","type":"function"},{"doc":"","ref":"Dmp.Diff.html#factor_out_prefixes/3","title":"Dmp.Diff.factor_out_prefixes/3","type":"function"},{"doc":"","ref":"Dmp.Diff.html#factor_out_suffixes/4","title":"Dmp.Diff.factor_out_suffixes/4","type":"function"},{"doc":"Given the original `text1`, and an encoded string which describes the\noperations required to transform `text1` into `text2`, compute the full diff.\n\n  * `text1 - Source string for the diff.\n  * `delta - Encoded delta text.\n\nReturns a difflist.\n\nRaises an `ArgumentError` if the encoded delta has invalid contents for the\ngiven text.","ref":"Dmp.Diff.html#from_delta/2","title":"Dmp.Diff.from_delta/2","type":"function"},{"doc":"Do the two texts share a substring which is at least half the length of\nthe longer text?\n\n  * `text1` - First string.\n  * `text2` - Second string.\n  * `deadline` - Unix timestamp (in milliseconds) at which to bail if not yet complete.\n\nThis speedup can produce non-minimal diffs.\n\nReturns a `half_match_result` 5-tuple, or `nil` if there was no match.\nReturns `nil` if `deadline` is zero (no time limit specified).","ref":"Dmp.Diff.html#half_match/3","title":"Dmp.Diff.half_match/3","type":"function"},{"doc":"Compute the Levenshtein distance of a diff--the number of inserted, deleted or\nsubstituted characters.","ref":"Dmp.Diff.html#levenshtein/1","title":"Dmp.Diff.levenshtein/1","type":"function"},{"doc":"Do a quick line-level diff on both strings, then rediff the parts for\ngreater accuracy.\n\n  * `text1` - Old string to be diffed.\n  * `text2` - New string to be diffed.\n  * `deadline` - Unix timestamp (in milliseconds) when the diff should be complete by.\n\nThis speedup can produce non-minimal diffs.\n\nReturns a difflist.","ref":"Dmp.Diff.html#line_mode/3","title":"Dmp.Diff.line_mode/3","type":"function"},{"doc":"Split two texts into a list of strings.\n\nReduce the texts to a string of hashes where each Unicode character\nrepresents one line.\n\n  * `text1` - First string.\n  * `text2` - Second string.\n\nReturns a tuple containing the encoded `text1`, the encoded `text2` and\nthe list of unique strings.  The zeroth element of the list of\nunique strings is intentionally blank.","ref":"Dmp.Diff.html#lines_to_chars/2","title":"Dmp.Diff.lines_to_chars/2","type":"function"},{"doc":"Find the differences between two texts.\n\n  * `text1` - Old string to be diffed.\n  * `text2` - New string to be diffed.\n  * `check_lines` - Speedup flag.  If `false`, then don't run a\n    line-level diff first to identify the changed areas. If `true`,\n    then run a faster slightly less optimal diff.\n  * `opts` - A options keyword list, `[]` to use the default options.\n\nMost of the time `check_lines` is wanted, so it defaults to `true`.\n\nReturns a difflist.","ref":"Dmp.Diff.html#main/4","title":"Dmp.Diff.main/4","type":"function"},{"doc":"Skips validation of options. Used internally by `Patch.apply`.","ref":"Dmp.Diff.html#main_/4","title":"Dmp.Diff.main_/4","type":"function"},{"doc":"Generate a pretty HTML report from a difflist.","ref":"Dmp.Diff.html#pretty_html/1","title":"Dmp.Diff.pretty_html/1","type":"function"},{"doc":"Given two strings, compute a score representing whether the internal\nboundary falls on logical boundaries.\n\nScores range from 6 (best) to 0 (worst).\n\n  * `one` - First string.\n  * `two` - Second string.\n\nScores are:\n\n  * 6 if `one` or `two` is an empty string.\n  * 5 if a blank line ends in `one` or a blank line starts in `two`.\n  * 4 if `one` ends, or `two` starts, with a newline.\n  * 3 if `one` ends in a punctuation and `two` starts with white space.\n  * 2 if `one` ends, or `two` starts, with white space.\n  * 1 if `one` ends, or `two` starts, with a non-alphanumeric.\n  * 0 otherwise","ref":"Dmp.Diff.html#semantic_score/2","title":"Dmp.Diff.semantic_score/2","type":"function"},{"doc":"iex> Diff.semantic_score(\"two is empty string\", \"\")\n    6\n\n    iex> Diff.semantic_score(\"one ends in blank line\\n\\n\", \"two\")\n    5\n\n    iex> Diff.semantic_score(\"one ends in new line\\n\", \"two\")\n    4\n\n    iex> Diff.semantic_score(\"one sentence.\", \" space before two\")\n    3\n\n    iex> Diff.semantic_score(\"one sentence.\", \"no space before two\")\n    1\n\n    iex> Diff.semantic_score(\"one ends with white space \", \"two\")\n    2\n\n    iex> Diff.semantic_score(\"one ends in 'punctuation'\", \"two\")\n    1\n\n    iex> Diff.semantic_score(\"one ends in middle of word\", \"two\")\n    0","ref":"Dmp.Diff.html#semantic_score/2-examples","title":"Examples - Dmp.Diff.semantic_score/2","type":"function"},{"doc":"","ref":"Dmp.Diff.html#sorted_half_match/2","title":"Dmp.Diff.sorted_half_match/2","type":"function"},{"doc":"Compute and return the source text of a diff (all equalities and deletions).","ref":"Dmp.Diff.html#text1/1","title":"Dmp.Diff.text1/1","type":"function"},{"doc":"Compute and return the destination text of a diff (all equalities and insertions).","ref":"Dmp.Diff.html#text2/1","title":"Dmp.Diff.text2/1","type":"function"},{"doc":"Crush a diff into an encoded string which describes the operations\nrequired to transform `text1` into `text2`.\n\nFor example, \"=3\t-2\t+ing\" means keep 3 chars, delete 2 chars, insert \"ing\".\n\nOperations are tab-separated.  Inserted text is escaped using %xx notation.","ref":"Dmp.Diff.html#to_delta/1","title":"Dmp.Diff.to_delta/1","type":"function"},{"doc":"|> iex [{:equal, \"abc\"}, {:delete, \"de\"}, {:insert, \"ing\"}] |> to_delta() |> IO.inspect()\n    \"=3\\t-2\\t+ing\"","ref":"Dmp.Diff.html#to_delta/1-examples","title":"Examples - Dmp.Diff.to_delta/1","type":"function"},{"doc":"Returns the diff tuple, or a \"nil\" pseudo-diff (with op `:nil`\nand empty text).","ref":"Dmp.Diff.html#undiff/1","title":"Dmp.Diff.undiff/1","type":"function"},{"doc":"Given `loc`, a location in `text1`, compute and return the equivalent location in\n`text2`.\n\n  * `diffs` - a difflist.\n  * `loc` - Location within `text1`.\n\nReturns location within `text2`.","ref":"Dmp.Diff.html#x_index/2","title":"Dmp.Diff.x_index/2","type":"function"},{"doc":"iex> Diff.main(\"The cat\", \"The big cat\") |> Diff.x_index(1)\n    1\n\n    iex> Diff.main(\"The cat\", \"The big cat\") |> Diff.x_index(4)\n    8","ref":"Dmp.Diff.html#x_index/2-examples","title":"Examples - Dmp.Diff.x_index/2","type":"function"},{"doc":"A list of diff operations, representing the difference between two text versions.\n\nA \"difflist\" is an Elixir list of \"diff\" tuples.\nHere is an example difflist:\n\n```\n[{:delete, \"Hello\"}, {:insert, \"Goodbye\"}, {:equal, \" world.\"}]\n```\n\nwhich means: delete \"Hello\", add \"Goodbye\" and keep \" world.\"","ref":"Dmp.Diff.html#t:difflist/0","title":"Dmp.Diff.difflist/0","type":"type"},{"doc":"","ref":"Dmp.Diff.html#t:expiry/0","title":"Dmp.Diff.expiry/0","type":"type"},{"doc":"","ref":"Dmp.Diff.html#t:first_pass_acc/0","title":"Dmp.Diff.first_pass_acc/0","type":"type"},{"doc":"The result of a successful `Diff.half_match/3` call.\n\nA tuple of five strings:\n\n  1. the prefix of `text1`\n  2. the suffix of `text1`\n  3. the prefix of `text2`\n  4. the suffix of `text2`\n  5. the common middle","ref":"Dmp.Diff.html#t:half_match_result/0","title":"Dmp.Diff.half_match_result/0","type":"type"},{"doc":"A diff's operation type. The operation `:nil` is used internally\nto indicate a nil value for the diff.","ref":"Dmp.Diff.html#t:op/0","title":"Dmp.Diff.op/0","type":"type"},{"doc":"","ref":"Dmp.Diff.html#t:options/0","title":"Dmp.Diff.options/0","type":"type"},{"doc":"The diff tuple, consisting of two elements: the operation\nand the associated text.","ref":"Dmp.Diff.html#t:t/0","title":"Dmp.Diff.t/0","type":"type"},{"doc":"Given a search string, find its best fuzzy match in a block of plain text.\nWeighted for both accuracy and location.","ref":"Dmp.Match.html","title":"Dmp.Match","type":"module"},{"doc":"Initialise the alphabet for the Bitap algorithm.\n\n`pattern` - The text to encode.\n\nReturns the map of character locations within the pattern.\nFrom the Wu and Manber paper:\n\n$$\\text{If } s\\xsb{i} .. s\\xsb{\\Sigma} \\text{ are the unique characters in the pattern,}$$\n$$\\text{and } p\\xsb{1} .. p\\xsb{m} \\text{ are the characters in the search pattern, then}$$\n$$S \\text{ is the bitarray where } S\\xsb{i}[r] = 1 \\text{ if } p\\xsb{r} = s\\xsb{i}$$\n\nFor best results the number of characters in `pattern` should less than or equal to the native bit size of an integer (32).","ref":"Dmp.Match.html#alphabet/1","title":"Dmp.Match.alphabet/1","type":"function"},{"doc":"Search for the best instance of `pattern` in `text` near `loc`, with errors,\nusing the Bitap algorithm.\n\nSee: [Fast Text Searching with Errors (Wu and Manber, 1991)](http://www.club.cc.cmu.edu/~ajo/docs/agrep.pdf).\n\n  * `text` - The text to search.\n  * `pattern` - The pattern to search for.\n  * `loc` - The location (zero-based index in `text`) to search around.\n  * `match_threshold` - At what point is no match declared\n    (0.0 = perfection, 1.0 = very loose, default = 0.5).\n  * `match_distance` - How far to search for a match (0 = exact location,\n    1000+ = broad match, default = 1000).\n  * `more_results` - Control which data is returned.\n\nIf `more_results` is false (the default), returns the index of the best\n  match closest to `loc` in `text`, or -1 if no suitable match was found.\n\nIf `more_results` is true, returns a tuple with three elements:\n  * The best match index in `text`, or -1 if no suitable match was found.\n  * The highest error level `d` that was searched.\n  * The adjusted match score at the returned index (a value less than\n    or equal to the `match_threshold`). If no match was found,\n    the score is 1.0.","ref":"Dmp.Match.html#bitap/6","title":"Dmp.Match.bitap/6","type":"function"},{"doc":"Compute and return a weighted score for a match with `e` errors and `x` location.\n\n  * `e` - Number of errors in match.\n  * `x` - Location of match.\n  * `loc` - Expected location of match.\n  * `pattern_length` - Length of pattern being sought.\n  * `match_distance` - How far to search for a match (0 = exact location, 1000+ = broad match).\n\nReturns overall score for match (0.0 = good, 1.0 = bad).","ref":"Dmp.Match.html#bitap_score/5","title":"Dmp.Match.bitap_score/5","type":"function"},{"doc":"Perform the bitap algorithm and calculate error score if a match is found.\n\n* `acc` - Accumulator tuple, with `best_loc`, `score_threshold`, `start`, and `rd` elements.\n* `constants` - Other constant values needed for calculations.\n\nUpdates the `rd` bitarray for the current error level `d` at the index `j`\n(representing the zero-based location `j - 1` in `text`), and\nthen tests for a match (an exact match if `d == 0` or a match with\n`d` errors).\n\nIf a match is found at position `j`, calculate the error score\n(based on the error level `d` and the distance from expected location).\nIf the score is lower than the current threshold, stop calculating the update\nif we have already gone below the minimum possible location,\nor continue the update, limiting the range of `j` (increasing the\n`start` value).","ref":"Dmp.Match.html#bitap_update/3","title":"Dmp.Match.bitap_update/3","type":"function"},{"doc":"The `j` index is decremented from the end of the text to the start of the text.\nSince the iteration is moving from high `j` to low, `bitap_update/3` does \"Lshift\"\noperations, not the \"Rshift\" operations in the Wu and Manber paper, and uses\nthe previous values that were set at `j + 1`, not `j`.\n\nHere the calculations are:\n\n$$R\\xsb{j}^d = \\begin{cases}\n  Lshift [ R\\xsb{j+1}^d ] \\text{ AND } S\\xsb{c} &\\text{if } d = 0 \\cr\n  Lshift [ R\\xsb{j+1}^d ] \\text{ AND } S\\xsb{c} \\text{ OR } Lshift [ R\\xsb{j}^{d-1} \\text{ OR } R\\xsb{j+1}^{d-1} ] \\text{ OR } R\\xsb{j+1}^{d-1} &\\text{otherwise}\n\\end{cases}$$\n\nversus in Wu and Manber's paper:\n\n$$R\\xsb{j}^d = \\begin{cases}\n  Rshift [ R\\xsb{j}^d ] \\text{ AND } S\\xsb{c} &\\text{if } d = 0 \\cr\n  Rshift [ R\\xsb{j}^d ] \\text{ AND } S\\xsb{c} \\text{ OR } Rshift [ R\\xsb{j}^{d-1} \\text{ OR } R\\xsb{j+1}^{d-1} ] \\text{ OR } R\\xsb{j}^{d-1} &\\text{otherwise}\n\\end{cases}$$","ref":"Dmp.Match.html#bitap_update/3-notes","title":"Notes - Dmp.Match.bitap_update/3","type":"function"},{"doc":"Look up a character in the alphabet and return its encoded bitmap.\n\n* `s` - An alphabet constructed with `alphabet/1`.\n* `ch` - A single character string.\n\nReturns a bitarray of positions in the `pattern` for the\ncharacter `ch`. In Wu and Manber:\n\n$$S\\xsb{c}$$","ref":"Dmp.Match.html#character_mask/2","title":"Dmp.Match.character_mask/2","type":"function"},{"doc":"Locate the best instance of `pattern` in `text` near `loc`.\n\n  * `text` - The text to search.\n  * `pattern` - The pattern to search for.\n  * `loc` - The location to search around.\n  * `opts` - A options keyword list, `[]` to use the default options.\n    In addition to the general parameters, the option `:more_results`,\n    if set to `true`, can be used to return more data. See `bitap/6`.\n\nReturns -1 if no match found.","ref":"Dmp.Match.html#main/4","title":"Dmp.Match.main/4","type":"function"},{"doc":"A bitarray encoding the locations of characters within the search pattern.\n\nThe index is the ordinal value of the character.","ref":"Dmp.Match.html#t:alpha/0","title":"Dmp.Match.alpha/0","type":"type"},{"doc":"A bitarray encoding possible match sequences of the search pattern within the text.\n\nThe index is the location of the text, plus one.","ref":"Dmp.Match.html#t:bitap_array/0","title":"Dmp.Match.bitap_array/0","type":"type"},{"doc":"","ref":"Dmp.Match.html#t:options/0","title":"Dmp.Match.options/0","type":"type"},{"doc":"Accumulator for `bitap_update/3`. A tuple with these elements:\n\n* `best_loc` - The value of `loc` with the lowest score found so far.\n* `score` - The lowest score found so far.\n* `start` - The lowest value of `j` that will be searched.\n* `rd` - The bitarray being calculated at the current error level `d`.","ref":"Dmp.Match.html#t:update_acc/0","title":"Dmp.Match.update_acc/0","type":"type"},{"doc":"Constants needed for `bitap_update/3`. A tuple with these elements:\n\n* `d` - Current error level.\n* `text` - Text to search.\n* `loc` -  The location (zero-based index in `text`) to search around.\n* `last_rd` - The bitarray from error level `d - 1`\n* `s` - The alphabet constructed from pattern being matched (see `alphabet/1`).\n* `match_mask` - A bitmap from the alphabet for the character at `loc` in `text`.\n* `overflow_mask` - A bitmap of all ones, of size `pattern_length`\n* `pattern_length` - The length of the pattern being matched.\n* `match_distance` - How far from `loc` that the search will be conducted.","ref":"Dmp.Match.html#t:update_constants/0","title":"Dmp.Match.update_constants/0","type":"type"},{"doc":"Adjustable parameters that control algorithm efficiency and accuracy.\n\n  * `:diff_timeout` - Number of seconds to map a diff before giving up (0 for infinity).\n  * `:diff_edit_cost` - Cost of an empty edit operation in terms of edit characters.\n  * `:match_max_bits` - The number of bits in an integer (default is expected 32).\n    This parameter controls the lengths of patterns used in matching and patch splitting.\n    Set `:match_max_bits` to 0 to disable patch splitting. To avoid long patches in\n    certain pathological cases, use 32.  Elixir supports arbitrarily large integers,\n    so we allow values of 64 and 128, as well as smaller values. Multiple short patches\n    (using native ints, `:match_max_bits` of 32 or less) should be much faster than long ones.\n  * `:match_threshold` - At what point is no match declared (0.0 = perfection, 1.0 = very loose).\n  * `:match_distance` - How far to search for a match (0 = exact location, 1000+ = broad match).\n    A match this many characters away from the expected location will add\n    1.0 to the score (0.0 is a perfect match).\n  * `:patch_delete_threshold` - When deleting a large block of text (over ~64 characters), how close do\n    the contents have to be to match the expected contents. (0.0 = perfection,\n    1.0 = very loose).  Note that `:match_threshold` controls how closely the\n    end points of a delete need to match.\n  * `:patch_margin` - Chunk size for context length. 4 is a good value.","ref":"Dmp.Options.html","title":"Dmp.Options","type":"module"},{"doc":"Returns an `Options` struct with good default values:","ref":"Dmp.Options.html#default/0","title":"Dmp.Options.default/0","type":"function"},{"doc":"iex> Options.default()\n    [\n      diff_edit_cost: 4,\n      diff_timeout: 1.0,\n      match_distance: 1000,\n      match_max_bits: 32,\n      match_threshold: 0.5,\n      patch_delete_threshold: 0.5,\n      patch_margin: 4\n    ]","ref":"Dmp.Options.html#default/0-examples","title":"Examples - Dmp.Options.default/0","type":"function"},{"doc":"Validates an `Options` list, raising an `ArgumentError` if it contains invalid values.\n\nIf `[]` is passed, `Options.default()` will be returned.","ref":"Dmp.Options.html#valid_options!/1","title":"Dmp.Options.valid_options!/1","type":"function"},{"doc":"iex> Options.valid_options!([])\n    [\n      diff_edit_cost: 4,\n      diff_timeout: 1.0,\n      match_distance: 1000,\n      match_max_bits: 32,\n      match_threshold: 0.5,\n      patch_delete_threshold: 0.5,\n      patch_margin: 4\n    ]\n\n    iex> Options.valid_options!(match_max_bits: -1)\n    ** (ArgumentError) Invalid options: match_max_bits (-1)","ref":"Dmp.Options.html#valid_options!/1-examples","title":"Examples - Dmp.Options.valid_options!/1","type":"function"},{"doc":"","ref":"Dmp.Options.html#t:option/0","title":"Dmp.Options.option/0","type":"type"},{"doc":"","ref":"Dmp.Options.html#t:t/0","title":"Dmp.Options.t/0","type":"type"},{"doc":"Apply a list of patches onto plain text. Use best effort to apply\npatch even when the underlying text doesn't match.","ref":"Dmp.Patch.html","title":"Dmp.Patch","type":"module"},{"doc":"Increase the context until it is unique,\nbut don't let the pattern expand beyond match_max_bits.\n\n  * `patch` - The `Patch` to grow.\n  * `text` - Source text.\n  * `patch_margin` - Chunk size for context length.\n  * `match_max_bits` - The number of bits in an integer (default is expected 32).\n\nReturns the updated `Patch`.","ref":"Dmp.Patch.html#add_context/4","title":"Dmp.Patch.add_context/4","type":"function"},{"doc":"","ref":"Dmp.Patch.html#add_diff_to_subpatch/4","title":"Dmp.Patch.add_diff_to_subpatch/4","type":"function"},{"doc":"","ref":"Dmp.Patch.html#add_other_diff_to_subpatch/4","title":"Dmp.Patch.add_other_diff_to_subpatch/4","type":"function"},{"doc":"Add some padding on text start and end so that edges can match something.\n\nIntended to be called only from within `Patch.apply`.\n\n  * `patches` - A patchlist..\n  * `patch_margin` - Chunk size for context length.\n\nReturns a tuple of the padded patchlist and the padding string added to each side.","ref":"Dmp.Patch.html#add_padding/2","title":"Dmp.Patch.add_padding/2","type":"function"},{"doc":"Merge a set of patches onto the text.  Return a patched text, as well\nas an array of true/false values indicating which patches were applied.\n\n  * `patches` - A patchlist.\n  * `text` - Text to apply patch to.\n  * `opts` - A options keyword list, `[]` to use the default options.\n\nReturns a tuple with two elements: the patched text, and a list of\nboolean values. Each boolean corresponds to a patch in the patchlist,\nand is `true` if a match was found for the corresponding patch.","ref":"Dmp.Patch.html#apply/2","title":"Dmp.Patch.apply/2","type":"function"},{"doc":"","ref":"Dmp.Patch.html#apply/3","title":"Dmp.Patch.apply/3","type":"function"},{"doc":"","ref":"Dmp.Patch.html#apply_match_diff/5","title":"Dmp.Patch.apply_match_diff/5","type":"function"},{"doc":"","ref":"Dmp.Patch.html#bad_match?/3","title":"Dmp.Patch.bad_match?/3","type":"function"},{"doc":"Compute a list of patches to turn `text1` into `text2`. `text1` will be derived\nfrom the provided diffs.\n\n  * `diffs` - A difflist from `text1` to `text2`.\n  * `opts` - A options keyword list, `[]` to use the default options.\n\nReturns a patchlist.","ref":"Dmp.Patch.html#from_diffs/2","title":"Dmp.Patch.from_diffs/2","type":"function"},{"doc":"Parse a textual representation of patches and return a patchlist.\n\nRaises an `ArgumentError` if the text has invalid contents.","ref":"Dmp.Patch.html#from_text/1","title":"Dmp.Patch.from_text/1","type":"function"},{"doc":"Deprecated\n\nCompute a list of patches to turn `text1` into `text2`. `text2` is ignored.\n`diffs` are the delta between text1 and text2.\n\n  * `text1` - Old text.\n  * `text2` - Ignored.\n  * `diffs` - A difflist from `text1` to `text2`.\n  * `opts` - A options keyword list, `[]` to use the default options.\n\nReturns a patchlist.","ref":"Dmp.Patch.html#from_texts_and_diffs/4","title":"Dmp.Patch.from_texts_and_diffs/4","type":"function"},{"doc":"This function can be called two ways. In either case the first argument,\n`a` is the original text (`text1`).\n\nThe second argument `b` has two cases:\n\n  * If `b` is a String `text2`, a difflist that turns `text1` into `text2` will be computed.\n  * If `b` is a difflist, it is the delta between `text1` and the target `text2`.\n  * `opts` - A options keyword list, `[]` to use the default options.\n\nReturns a patchlist.","ref":"Dmp.Patch.html#make/3","title":"Dmp.Patch.make/3","type":"function"},{"doc":"Look through the patches and break up any which are longer than the\nmaximum limit of the match algorithm.\n\nIntended to be called only from within `Patch.apply`.\n\n  * `patches` - A patchlist.\n  * `patch_margin` - Chunk size for context length.\n  * `match_max_bits` - The number of bits in an int (default 32).\n\nReturns the updated patchlist.","ref":"Dmp.Patch.html#split_max/3","title":"Dmp.Patch.split_max/3","type":"function"},{"doc":"","ref":"Dmp.Patch.html#subpatch_loop/3","title":"Dmp.Patch.subpatch_loop/3","type":"function"},{"doc":"Return the textual representation of a patchlist.","ref":"Dmp.Patch.html#to_text/1","title":"Dmp.Patch.to_text/1","type":"function"},{"doc":"","ref":"Dmp.Patch.html#t:apply_loop_acc/0","title":"Dmp.Patch.apply_loop_acc/0","type":"type"},{"doc":"","ref":"Dmp.Patch.html#t:options/0","title":"Dmp.Patch.options/0","type":"type"},{"doc":"","ref":"Dmp.Patch.html#t:patchlist/0","title":"Dmp.Patch.patchlist/0","type":"type"},{"doc":"","ref":"Dmp.Patch.html#t:t/0","title":"Dmp.Patch.t/0","type":"type"},{"doc":"Java.String- and Javascript-compatible functions missing in Elixir's `String` module.","ref":"Dmp.StringUtils.html","title":"Dmp.StringUtils","type":"module"},{"doc":"Returns the index within this string of the first occurrence of the specified substring,\nor -1 if there is no such occurence.","ref":"Dmp.StringUtils.html#index_of/2","title":"Dmp.StringUtils.index_of/2","type":"function"},{"doc":"ies> StringUtils.index_of(\"\", \"\")\n    0\n\n    iex> StringUtils.index_of(\"abracadabra\", \"b\")\n    1\n\n    iex> StringUtils.index_of(\"abracadabra\", \"f\")\n    -1","ref":"Dmp.StringUtils.html#index_of/2-examples","title":"Examples - Dmp.StringUtils.index_of/2","type":"function"},{"doc":"Returns the index within this string of the first occurrence of the specified substring,\nstarting the search at the specified index, or -1 if there is no such occurence.","ref":"Dmp.StringUtils.html#index_of/3","title":"Dmp.StringUtils.index_of/3","type":"function"},{"doc":"iex> StringUtils.index_of(\"abracadabra\", \"\", 2)\n    2\n\n    iex> StringUtils.index_of(\"abracadabra\", \"\", 100)\n    11\n\n    iex> StringUtils.index_of(\"abracadabra\", \"b\", 2)\n    8\n\n    iex> StringUtils.index_of(\"abracadabra\", \"f\", 2)\n    -1","ref":"Dmp.StringUtils.html#index_of/3-examples","title":"Examples - Dmp.StringUtils.index_of/3","type":"function"},{"doc":"Returns the index within this string of the last occurrence of the specified substring,\nor -1 if there is no such occurence.","ref":"Dmp.StringUtils.html#last_index_of/2","title":"Dmp.StringUtils.last_index_of/2","type":"function"},{"doc":"ies> StringUtils.last_index_of(\"\", \"\")\n    0\n\n    ies> StringUtils.last_index_of(\"abracadabra\", \"\")\n    11\n\n    iex> StringUtils.last_index_of(\"abracadabra\", \"b\")\n    8\n\n    iex> StringUtils.last_index_of(\"abracadabra\", \"f\")\n    -1","ref":"Dmp.StringUtils.html#last_index_of/2-examples","title":"Examples - Dmp.StringUtils.last_index_of/2","type":"function"},{"doc":"Returns the index within this string of the last occurrence of the specified substring,\nstarting the search at the specified index, or -1 if there is no such occurence.","ref":"Dmp.StringUtils.html#last_index_of/3","title":"Dmp.StringUtils.last_index_of/3","type":"function"},{"doc":"iex> StringUtils.index_of(\"abracadabra\", \"b\", 5)\n    8\n\n    iex> StringUtils.index_of(\"abracadabra\", \"d\", 9)\n    -1\n\n    iex> StringUtils.last_index_of(\"abcdefghijk\", \"fgh\", 5)\n    5\n\n    ies> StringUtils.last_index_of(\"abcdefghijk\", \"\", 5)\n    11","ref":"Dmp.StringUtils.html#last_index_of/3-examples","title":"Examples - Dmp.StringUtils.last_index_of/3","type":"function"},{"doc":"Returns a new string that is a substring of this string.\n\nThe substring begins with the character at the specified index\nand extends to the end of this string.","ref":"Dmp.StringUtils.html#substring/2","title":"Dmp.StringUtils.substring/2","type":"function"},{"doc":"iex> StringUtils.substring(\"abracadabra\", 6)\n    \"dabra\"\n\n    iex> StringUtils.substring(\"abracadabra\", 12)\n    \"\"","ref":"Dmp.StringUtils.html#substring/2-examples","title":"Examples - Dmp.StringUtils.substring/2","type":"function"},{"doc":"Returns a new string that is a substring of this string.\n\nThe substring begins at the specified `begin_index` and extends to the\ncharacter at index `end_index - 1`. Thus the length of the substring\nis `end_index - begin_index`.","ref":"Dmp.StringUtils.html#substring/3","title":"Dmp.StringUtils.substring/3","type":"function"},{"doc":"iex> StringUtils.substring(\"abracadabra\", 2, 6)\n    \"raca\"\n\n    iex> StringUtils.substring(\"abracadabra\", 2, 12)\n    \"racadabra\"","ref":"Dmp.StringUtils.html#substring/3-examples","title":"Examples - Dmp.StringUtils.substring/3","type":"function"},{"doc":"Unescape selected chars for compatability with JavaScript's `encodeURI`.\n\nIn speed critical applications this could be dropped since the\nreceiving application will certainly decode these fine.\nNote that this function is case-sensitive.  Thus \"%3f\" would not be\nunescaped.  But this is ok because it is only called with the output of\n`StringUtils.uri_encode` which returns uppercase hex.","ref":"Dmp.StringUtils.html#unescape_for_encode_uri_compatability/1","title":"Dmp.StringUtils.unescape_for_encode_uri_compatability/1","type":"function"},{"doc":"iex> StringUtils.unescape_for_encode_uri_compatability(\"%3F\")\n    \"?\"\n\n    iex> StringUtils.unescape_for_encode_uri_compatability(\"%3f\")\n    \"%3f\"\n\n    iex> StringUtils.unescape_for_encode_uri_compatability(\"%24\")\n    \"$\"","ref":"Dmp.StringUtils.html#unescape_for_encode_uri_compatability/1-examples","title":"Examples - Dmp.StringUtils.unescape_for_encode_uri_compatability/1","type":"function"},{"doc":"A URI encoding, but with spaces and asterisks left as is, for use with diffs.","ref":"Dmp.StringUtils.html#uri_encode/1","title":"Dmp.StringUtils.uri_encode/1","type":"function"},{"doc":"iex> StringUtils.uri_encode(\"(\")\n    \"%28\"\n\n    iex> StringUtils.uri_encode(\" \")\n    \" \"\n\n    iex> StringUtils.uri_encode(\"*\")\n    \"*\"\n\n    iex> StringUtils.uri_encode(\"[]\")\n    \"%5B%5D\"","ref":"Dmp.StringUtils.html#uri_encode/1-examples","title":"Examples - Dmp.StringUtils.uri_encode/1","type":"function"},{"doc":"# diff_match_patch\n\nA translation of Google's public-domain\n[diff_match_patch](https://github.com/google/diff-match-patch) code into pure Elixir.\n\nFor information about the original Google project and its application,\nsee that repository's [wiki pages](https://github.com/google/diff-match-patch/wiki).\n\nReferences from the Google project:\n\n* diff: [An O(ND) Difference Algorithm and Its Variations (Meyers, 1986)](http://www.xmailserver.org/diff2.pdf)\n* match: [Fast Text Searching with Errors (Wu and Manber, 1991)](http://www.club.cc.cmu.edu/~ajo/docs/agrep.pdf)\n\nPrior art on the BEAM:\n\n* diffy: [A Diff, Match and Patch implementation for Erlang](https://github.com/zotonic/diffy)","ref":"readme.html","title":"Diff Match Patch","type":"extras"},{"doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `diff_match_patch` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:diff_match_patch, \"~> 0.2.0\"}\n  ]\nend\n```\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#installation","title":"Installation - Diff Match Patch","type":"extras"},{"doc":"1. Find a **match** for a pattern inside at text using fuzzy search. The\n`match_threshold` option determines the exactness required:\n\n```\nloc = Dmp.Match.main(\n    \"I am the very model of a modern major general.\",\n    \" that berry \",\n    5,\n     match_threshold: 0.7)\n```\n\nThe value of `loc` is 4, finding the closest match at the string starting \" the very\".\nIf no close match is found, the `loc` returned is -1.\n\n2. Create a **diff** between two texts:\n\n```\ndiffs = Dmp.Diff.main(\n    \"The quick brown fox jumps over the lazy dog.\",\n    \"That quick brown fox jumped over a lazy dog.\")\n```\n\n3. Create a **patch** from the the original text and the diff we just created.\nThe patch can also be created directly from the two texts. These\ntwo patches are equal:\n\n```\npatches1 = Dmp.Patch.make(\n    \"The quick brown fox jumps over the lazy dog.\", diffs)\npatches2 = Dmp.Patch.make(\n    \"The quick brown fox jumps over the lazy dog.\",\n    \"That quick brown fox jumped over a lazy dog.\")\n```\n\n4. Apply the patch to a third text:\n\n```\n{new_text, matched} = Patch.apply(patches, \"The quick red rabbit jumps over the tired tiger.\")\nIO.puts(new_text)\n```\n\nThe `new_text` prints out as \"That quick red rabbit jumped over a tired tiger.\"\nThe `matched` value is a list of booleans, showing whether the results of whether\nmatches were made between the (expanded) list of patches and the third text.","ref":"readme.html#usage","title":"Usage - Diff Match Patch","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}